import unittest
from json import loads
from registrantalert import Response, ErrorMessage


_json_response_ok_empty = '''{
   "domainsCount": 0,
    "domainsList": []
}'''

_json_response_ok = '''{
   "domainsCount": 2,
   "domainsList": [
        {
            "domainName": "villas-montego-bay.com",
            "date": "2021-09-13",
            "action": "added"
        },
        {
            "domainName": "airbnb-checkin-rooms1511.com",
            "date": "2021-09-14",
            "action": "dropped"
        }
    ]
}'''

_json_response_error = '''{
    "code": 403,
    "messages": "Access restricted. Check credits balance or enter the correct API key."
}'''


class TestModel(unittest.TestCase):

    def test_response_parsing(self):
        response = loads(_json_response_ok)
        parsed = Response(response)
        self.assertEqual(parsed.domains_count, response['domainsCount'])
        self.assertIsInstance(parsed.domains_list, list)
        self.assertEqual(
            parsed.domains_list[0].domain_name,
            response['domainsList'][0]['domainName'])

    def test_ok_with_dates(self):
        response = loads(_json_response_ok)
        parsed = Response(response)
        self.assertEqual(parsed.domains_count, response['domainsCount'])
        self.assertIsInstance(parsed.domains_list, list)
        self.assertEqual(
            parsed.domains_list[1].domain_name,
            response['domainsList'][1]['domainName'])
        self.assertEqual(
            parsed.domains_list[1].date.strftime(
                "%Y-%m-%d"),
            ''.join(response['domainsList'][1]['date']
                    .rsplit(':', 1))
        )
        self.assertEqual(
            parsed.domains_list[1].action,
            ''.join(response['domainsList'][1]['action']
                    .rsplit(':', 1))
        )

    def test_error_parsing(self):
        error = loads(_json_response_error)
        parsed_error = ErrorMessage(error)
        self.assertEqual(parsed_error.code, error['code'])
        self.assertEqual(parsed_error.message, error['messages'])



class Fields:
    name_servers = 'NameServers'
    whois_server = 'WhoisServer'
    registrant_contact_city = 'RegistrantContact.City'
    registrant_contact_country = 'RegistrantContact.Country'
    registrant_contact_email = 'RegistrantContact.Email'
    registrant_contact_fax = 'RegistrantContact.Fax'
    registrant_contact_fax_extension = 'RegistrantContact.FaxExtension'
    registrant_contact_name = 'RegistrantContact.Name'
    registrant_contact_organization = 'RegistrantContact.Organization'
    registrant_contact_postal_code = 'RegistrantContact.PostalCode'
    registrant_contact_state = 'RegistrantContact.State'
    registrant_contact_street_address = 'RegistrantContact.StreetAddress'
    registrant_contact_telephone = 'RegistrantContact.Telephone'
    registrant_contact_telephone_extension = 'RegistrantContact.TelephoneExtension'
    admin_contact_city = 'AdminContact.City'
    admin_contact_country = 'AdminContact.Country'
    admin_contact_email = 'AdminContact.Email'
    admin_contact_fax = 'AdminContact.Fax'
    admin_contact_fax_extension = 'AdminContact.FaxExtension'
    admin_contact_name = 'AdminContact.Name'
    admin_contact_organization = 'AdminContact.Organization'
    admin_contact_postal_code = 'AdminContact.PostalCode'
    admin_contact_state = 'AdminContact.State'
    admin_contact_street_address = 'AdminContact.StreetAddress'
    admin_contact_telephone = 'AdminContact.Telephone'
    admin_contact_telephone_extension = 'AdminContact.TelephoneExtension'
    billing_contact_city = 'BillingContact.City'
    billing_contact_country = 'BillingContact.Country'
    billing_contact_email = 'BillingContact.Email'
    billing_contact_fax = 'BillingContact.Fax'
    billing_contact_fax_extension = 'BillingContact.FaxExtension'
    billing_contact_name = 'BillingContact.Name'
    billing_contact_organization = 'BillingContact.Organization'
    billing_contact_postal_code = 'BillingContact.PostalCode'
    billing_contact_state = 'BillingContact.State'
    billing_contact_street_address = 'BillingContact.StreetAddress'
    billing_contact_telephone = 'BillingContact.Telephone'
    billing_contact_telephone_extension = 'BillingContact.TelephoneExtension'
    tech_contact_city = 'TechContact.City'
    tech_contact_country = 'TechContact.Country'
    tech_contact_email = 'TechContact.Email'
    tech_contact_fax = 'TechContact.Fax'
    tech_contact_fax_extension = 'TechContact.FaxExtension'
    tech_contact_name = 'TechContact.Name'
    tech_contact_organization = 'TechContact.Organization'
    tech_contact_postal_code = 'TechContact.PostalCode'
    tech_contact_state = 'TechContact.State'
    tech_contact_street_address = 'TechContact.StreetAddress'
    tech_contact_telephone = 'TechContact.Telephone'
    tech_contact_telephone_extension = 'TechContact.TelephoneExtension'

    @staticmethod
    def keys() -> list:
        return list(
            filter(lambda x: not x.startswith('_'), Fields.__dict__)
        )

    @staticmethod
    def values() -> list:
        return [Fields.__dict__[k] for k in Fields.keys()]



__all__ = ['Client', 'ErrorMessage', 'RegistrantAlertApiError', 'ApiAuthError',
           'HttpApiError', 'EmptyApiKeyError', 'ParameterError',
           'ResponseError', 'BadRequestError', 'UnparsableApiResponseError',
           'ApiRequester', 'Domain', 'Response', 'Fields']

from .client import Client
from .net.http import ApiRequester
from .models.request import Fields
from .models.response import ErrorMessage, Response, Domain
from .exceptions.error import RegistrantAlertApiError, ParameterError, \
    EmptyApiKeyError, ResponseError, UnparsableApiResponseError, \
    ApiAuthError, BadRequestError, HttpApiError



import datetime
import os
import unittest
from registrantalert import Client, Fields
from registrantalert import ParameterError, ApiAuthError


class TestClient(unittest.TestCase):
    """
    Final integration tests without mocks.

    Active API_KEY is required.
    """

    def setUp(self) -> None:
        self.client = Client(os.getenv('API_KEY'))
        self.correct_basic_terms = {
            'include': ['app'],
            'exclude': ['blog']
        }

        self.correct_advanced_terms = [
            {
                'field': Fields.registrant_contact_organization,
                'term': "Airbnb, Inc."
            }
        ]

        self.incorrect_basic_terms = {
            'include': []
        }

        self.incorrect_advanced_terms = [
            {
                'field': 'abrakadabra'
            }
        ]

    def test_get_correct_data(self):
        response = self.client.data(
            basic_terms=self.correct_basic_terms,
            mode=Client.PURCHASE_MODE,
        )
        self.assertIsNotNone(response.domains_count)

    def test_extra_parameters(self):
        response = self.client.data(
            basic_terms=self.correct_basic_terms,
            mode=Client.PREVIEW_MODE,
            since_date=datetime.date.today() - datetime.timedelta(days=10),
            created_date_from=datetime.date(year=2019, month=1, day=1)
        )
        self.assertIsNotNone(response.domains_count)

    def test_empty_terms(self):
        with self.assertRaises(ParameterError):
            self.client.data()

    def test_empty_api_key(self):
        with self.assertRaises(ParameterError):
            client = Client('')
            client.data(basic_terms=self.correct_basic_terms)

    def test_incorrect_api_key(self):
        client = Client('at_00000000000000000000000000000')
        with self.assertRaises(ApiAuthError):
            client.data(basic_terms=self.correct_basic_terms)

    def test_raw_data(self):
        response = self.client.raw_data(
            basic_terms=self.correct_basic_terms,
            response_format=Client.XML_FORMAT)
        self.assertTrue(response.startswith('<?xml'))

    def test_advanced(self):
        response = self.client.data(
            advanced_terms=self.correct_advanced_terms
        )
        self.assertIsNotNone(response.domains_count)

    def test_incorrect_basic(self):
        with self.assertRaises(ParameterError):
            self.client.data(basic_terms=self.incorrect_basic_terms)

    def test_incorrect_advanced(self):
        with self.assertRaises(ParameterError):
            self.client.data(advanced_terms=self.incorrect_advanced_terms)

    def test_preview(self):
        response = self.client.preview(basic_terms=self.correct_basic_terms)
        self.assertGreater(response.domains_count, 0)

    def test_purchase(self):
        response = self.client.purchase(basic_terms=self.correct_basic_terms)
        self.assertGreater(len(response.domains_list), 0)

    def test_incorrect_date(self):
        with self.assertRaises(ParameterError):
            self.client.data(basic_terms=self.correct_basic_terms,
                             punicode='true',
                             updated_date_from='2020-01-01',
                             expired_date_to='19-19-19')

    def test_punycode(self):
        with self.assertRaises(ParameterError):
            self.client.data(advanced_terms=self.correct_advanced_terms,
                             punycode='not sure')

    def test_output(self):
        with self.assertRaises(ParameterError):
            self.client.data(advanced_terms=self.correct_advanced_terms,
                             output_format='yaml')


if __name__ == '__main__':
    unittest.main()



import datetime
from json import loads, JSONDecodeError
import re

from .net.http import ApiRequester
from .models.response import Response
from .models.request import Fields
from .exceptions.error import ParameterError, EmptyApiKeyError, \
    UnparsableApiResponseError


class Client:
    __default_url = "https://registrant-alert.whoisxmlapi.com/api/v2"
    _api_requester: ApiRequester or None
    _api_key: str

    _re_api_key = re.compile(r'^at_[a-z0-9]{29}$', re.IGNORECASE)
    _SUPPORTED_FORMATS = ['json', 'xml']

    _PARSABLE_FORMAT = 'json'

    JSON_FORMAT = 'json'
    XML_FORMAT = 'xml'
    PREVIEW_MODE = 'preview'
    PURCHASE_MODE = 'purchase'

    __DATETIME_OR_NONE_MSG = 'Value should be None or an instance of ' \
                             'datetime.date'

    def __init__(self, api_key: str, **kwargs):
        """
        :param api_key: str: Your API key.
        :key base_url: str: (optional) API endpoint URL.
        :key timeout: float: (optional) API call timeout in seconds
        """

        self._api_key = ''

        self.api_key = api_key

        if 'base_url' not in kwargs:
            kwargs['base_url'] = Client.__default_url

        self.api_requester = ApiRequester(**kwargs)

    @property
    def api_key(self) -> str:
        return self._api_key

    @api_key.setter
    def api_key(self, value: str):
        self._api_key = Client._validate_api_key(value)

    @property
    def api_requester(self) -> ApiRequester or None:
        return self._api_requester

    @api_requester.setter
    def api_requester(self, value: ApiRequester):
        self._api_requester = value

    @property
    def base_url(self) -> str:
        return self._api_requester.base_url

    @base_url.setter
    def base_url(self, value: str or None):
        if value is None:
            self._api_requester.base_url = Client.__default_url
        else:
            self._api_requester.base_url = value

    @property
    def timeout(self) -> float:
        return self._api_requester.timeout

    @timeout.setter
    def timeout(self, value: float):
        self._api_requester.timeout = value

    def preview(self, **kwargs) -> Response:
        """
        Get parsed API response as a `Response` instance.
        Mode = `preview`

        :key basic_terms: Required if advanced_terms aren't specified.
                Dictionary. Take a look at API documentation for the format
        :key advanced_terms: Required if basic_terms aren't specified
                List. Take a look at API documentation for the format
        :key since_date: Optional. datetime.date. Yesterday's date by default.
        :key punycode: Optional. Boolean. Default value is `True`
        :key include_audit_dates: Optional. Boolean. Default value is `False`
        :key created_date_from: Optional. datetime.date.
        :key created_date_to: Optional. datetime.date.
        :key updated_date_from: Optional. datetime.date.
        :key updated_date_to: Optional. datetime.date.
        :key expired_date_from: Optional. datetime.date.
        :key expired_date_to: Optional. datetime.date.
        :return: `Response` instance
        :raises ConnectionError:
        :raises RegistrantAlertApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter's value
        """

        kwargs['mode'] = Client.PREVIEW_MODE
        return self.data(**kwargs)

    def purchase(self, **kwargs):
        """
        Get parsed API response as a `Response` instance.
        Mode = `purchase`

        :key basic_terms: Required if advanced_terms aren't specified.
                Dictionary. Take a look at API documentation for the format
        :key advanced_terms: Required if basic_terms aren't specified
                List. Take a look at API documentation for the format
        :key since_date: Optional. datetime.date. Yesterday's date by default.
        :key punycode: Optional. Boolean. Default value is `True`
        :key created_date_from: Optional. datetime.date.
        :key created_date_to: Optional. datetime.date.
        :key updated_date_from: Optional. datetime.date.
        :key updated_date_to: Optional. datetime.date.
        :key expired_date_from: Optional. datetime.date.
        :key expired_date_to: Optional. datetime.date.
        :return: `Response` instance
        :raises ConnectionError:
        :raises RegistrantAlertApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter's value
        """
        kwargs['mode'] = Client.PURCHASE_MODE
        return self.data(**kwargs)

    def data(self, **kwargs) -> Response:
        """
        Get parsed API response as a `Response` instance.

        :key basic_terms: Required if advanced_terms aren't specified.
                Dictionary. Take a look at API documentation for the format
        :key advanced_terms: Required if basic_terms aren't specified
                List. Take a look at API documentation for the format
        :key mode: Optional. Supported options - `Client.PREVIEW_MODE` and
                `Client.PURCHASE_MODE`. Default is `Client.PREVIEW_MODE`
        :key since_date: Optional. datetime.date. Yesterday's date by default.
        :key punycode: Optional. Boolean. Default value is `True`
        :key created_date_from: Optional. datetime.date.
        :key created_date_to: Optional. datetime.date.
        :key updated_date_from: Optional. datetime.date.
        :key updated_date_to: Optional. datetime.date.
        :key expired_date_from: Optional. datetime.date.
        :key expired_date_to: Optional. datetime.date.
        :return: `Response` instance
        :raises ConnectionError:
        :raises RegistrantAlertApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter's value
        """

        kwargs['response_format'] = Client._PARSABLE_FORMAT

        response = self.raw_data(**kwargs)
        try:
            parsed = loads(str(response))
            if 'domainsCount' in parsed:
                return Response(parsed)
            raise UnparsableApiResponseError(
                "Could not find the correct root element.", None)
        except JSONDecodeError as error:
            raise UnparsableApiResponseError("Could not parse API response", error)

    def raw_data(self, **kwargs) -> str:
        """
        Get raw API response.

        :key basic_terms: Required if advanced_terms aren't specified.
                Dictionary. Take a look at API documentation for the format
        :key advanced_terms: Required if basic_terms aren't specified
                List. Take a look at API documentation for the format
        :key mode: Optional. Supported options - `Client.PREVIEW_MODE` and
                `Client.PURCHASE_MODE`. Default is `Client.PREVIEW_MODE`
        :key since_date: Optional. datetime.date. Yesterday's date by default.
        :key punycode: Optional. Boolean. Default value is `True`
        :key created_date_from: Optional. datetime.date.
        :key created_date_to: Optional. datetime.date.
        :key updated_date_from: Optional. datetime.date.
        :key updated_date_to: Optional. datetime.date.
        :key expired_date_from: Optional. datetime.date.
        :key expired_date_to: Optional. datetime.date.
        :key response_format: Optional. use constants
                JSON_FORMAT and XML_FORMAT
        :return: str
        :raises ConnectionError:
        :raises RegistrantAlertApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter's value
        """

        if self.api_key == '':
            raise EmptyApiKeyError('')

        if 'basic_terms' in kwargs:
            basic_terms = Client._validate_basic_terms(kwargs['basic_terms'])
        else:
            basic_terms = None

        if 'advanced_terms' in kwargs:
            advanced_terms = Client._validate_advanced_terms(
                kwargs['advanced_terms'])
        else:
            advanced_terms = None

        if not advanced_terms and not basic_terms:
            raise ParameterError(
                "Required one from basic_terms and advanced_terms")

        if 'output_format' in kwargs:
            kwargs['response_format'] = kwargs['output_format']
        if 'response_format' in kwargs:
            response_format = Client._validate_response_format(
                kwargs['response_format'])
        else:
            response_format = Client._PARSABLE_FORMAT

        if 'since_date' in kwargs:
            since_date = Client._validate_date(kwargs['since_date'])
        else:
            since_date = Client._validate_date(datetime.date.today() - datetime.timedelta(days=1))

        if 'punycode' in kwargs:
            punycode = Client._validate_punycode(kwargs['punycode'])
        else:
            punycode = True

        if 'mode' in kwargs:
            mode = Client._validate_mode(kwargs['mode'])
        else:
            mode = Client.PREVIEW_MODE

        if 'created_date_from' in kwargs:
            created_date_from = Client._validate_date(
                kwargs['created_date_from']
            )
        else:
            created_date_from = None

        if 'created_date_to' in kwargs:
            created_date_to = Client._validate_date(
                kwargs['created_date_to']
            )
        else:
            created_date_to = None

        if 'updated_date_from' in kwargs:
            updated_date_from = Client._validate_date(
                kwargs['updated_date_from']
            )
        else:
            updated_date_from = None

        if 'updated_date_to' in kwargs:
            updated_date_to = Client._validate_date(
                kwargs['updated_date_to']
            )
        else:
            updated_date_to = None

        if 'expired_date_from' in kwargs:
            expired_date_from = Client._validate_date(
                kwargs['expired_date_from']
            )
        else:
            expired_date_from = None

        if 'expired_date_to' in kwargs:
            expired_date_to = Client._validate_date(
                kwargs['expired_date_to']
            )
        else:
            expired_date_to = None

        return self._api_requester.post(self._build_payload(
            self.api_key,
            basic_terms,
            advanced_terms,
            mode,
            since_date,
            punycode,
            response_format,
            created_date_from,
            created_date_to,
            updated_date_from,
            updated_date_to,
            expired_date_from,
            expired_date_to,
        ))

    @staticmethod
    def _validate_api_key(api_key) -> str:
        if Client._re_api_key.search(
                str(api_key)
        ) is not None:
            return str(api_key)
        else:
            raise ParameterError("Invalid API key format.")

    @staticmethod
    def _validate_basic_terms(value) -> dict:
        include, exclude = [], []
        if value is None:
            raise ParameterError("Terms list cannot be None.")
        elif type(value) is dict:
            if 'include' in value:
                include = list(map(lambda s: str(s), value['include']))
                include = list(
                    filter(lambda s: s is not None and len(s) > 0, include))
                if 4 <= len(include) <= 1:
                    raise ParameterError("Include terms list must include "
                                         "from 1 to 4 terms.")
            if 'exclude' in value:
                exclude = list(map(lambda s: str(s), value['exclude']))
                exclude = list(
                    filter(lambda s: s is not None and len(s) > 0, exclude))
                if 4 <= len(exclude) <= 0:
                    raise ParameterError("Exclude terms list must include "
                                         "from 0 to 4 terms.")
            if include:
                return {'include': include, 'exclude': exclude}

        raise ParameterError("Expected a dict with 2 lists of strings.")

    @staticmethod
    def _validate_advanced_terms(value) -> list:
        if value is None:
            raise ParameterError("Terms list cannot be None.")
        elif type(value) is list:
            if 4 <= len(value) < 1:
                raise ParameterError(
                    "Terms list must include form 1 to 4 items.")
            for item in value:
                if 'field' not in item or 'term' not in item:
                    raise ParameterError(
                        "Invalid advanced search terms format."
                        "The 'field' or 'term' is missing.")
                if item['field'] not in Fields.values():
                    raise ParameterError("Unknown field name.")
                if item['term'] is None or type(item['term']) is not str \
                        or len(item['term']) < 2:
                    raise ParameterError("Term should be non-empty string.")
            return value

        raise ParameterError("Expected a list of pairs field <-> term.")

    @staticmethod
    def _validate_response_format(value: str):
        if value.lower() in [Client.JSON_FORMAT, Client.XML_FORMAT]:
            return value.lower()

        raise ParameterError(
            f"Response format must be {Client.JSON_FORMAT} "
            f"or {Client.XML_FORMAT}")

    @staticmethod
    def _validate_mode(value: str):
        if value.lower() in [Client.PREVIEW_MODE, Client.PURCHASE_MODE]:
            return value.lower()

        raise ParameterError(
            f"Mode must be {Client.PREVIEW_MODE} or {Client.PURCHASE_MODE}")

    @staticmethod
    def _validate_punycode(value: bool):
        if value in [True, False]:
            return value

        raise ParameterError(
            "Punycode parameter value must be True or False")

    @staticmethod
    def _validate_date(value: datetime.date or None):
        if value is None or isinstance(value, datetime.date):
            return str(value)

        raise ParameterError(Client.__DATETIME_OR_NONE_MSG)

    @staticmethod
    def _build_payload(
            api_key,
            basic_terms,
            advanced_terms,
            mode,
            since_date,
            punycode,
            response_format,
            created_date_from,
            created_date_to,
            updated_date_from,
            updated_date_to,
            expired_date_from,
            expired_date_to,
    ) -> dict:
        tmp = {
            'apiKey': api_key,
            'basicSearchTerms': basic_terms,
            'advancedSearchTerms': advanced_terms,
            'mode': mode,
            'sinceDate': since_date,
            'punycode': punycode,
            'responseFormat': response_format,
            'createdDateFrom': created_date_from,
            'createdDateTo': created_date_to,
            'updatedDateFrom': updated_date_from,
            'updatedDateTo': updated_date_to,
            'expiredDateFrom': expired_date_from,
            'expiredDateTo': expired_date_to,
        }

        payload = {}
        for k, v in tmp.items():
            if v is not None:
                payload[k] = v
        return payload
